@@ -275,6 +275,7 @@ async def submit (request):
     reader = await request.multipart()
     job_options = {}
     input_files = []
+    cc_cohorts_path = None
     while True:
         part = await reader.next()
         if not part: 
@@ -288,6 +289,10 @@ async def submit (request):
                 wf.write(await part.read())
         elif part.name == 'options':
             job_options = await part.json()
+        elif part.name == 'casecontrol':
+            cc_cohorts_path = os.path.join(job_dir, part.filename)
+            with open(cc_cohorts_path,'wb') as wf:
+                wf.write(await part.read())
     input_fnames = [fp.filename for fp in input_files]
     run_name = input_fnames[0]
     if len(input_fnames) > 1:
@@ -348,6 +353,8 @@ async def submit (request):
         run_args.append('--writeadmindb')
         run_args.extend(['--jobid', job_id])
     run_args.append('--temp-files')
+    if cc_cohorts_path is not None:
+        run_args.extend(['--module-option',f'casecontrol.cohorts={cc_cohorts_path}'])
     global job_queue
     global run_jobs_info
     job_ids = run_jobs_info['job_ids']
@@ -477,6 +484,8 @@ async def get_job (request, job_id):
                 if os.path.exists(os.path.join(job_dir, p)) == False:
                     report_exist = False
                     break
+            if os.path.exists(os.path.join(job_dir, job_id + '.report_being_generated.' + report_type)):
+                report_exist = False
             if report_exist:
                 existing_reports.append(report_type)
                 global report_generation_ps
@@ -592,11 +601,8 @@ def get_valid_report_types():
     return report_types
 
 async def get_report_types(request):
-    global cfl
-    default_reporter = cfl.get_cravat_conf_value('reporter')
-    default_type = default_reporter.split('reporter')[0]
     valid_types = get_valid_report_types()
-    return web.json_response({'valid': valid_types, 'default': default_type})
+    return web.json_response({'valid': valid_types})
 
 async def generate_report(request):
     global filerouter
@@ -608,8 +614,13 @@ async def generate_report(request):
     if job_id not in report_generation_ps:
         report_generation_ps[job_id] = {}
     report_generation_ps[job_id][report_type] = True
+    tmp_flag_path = os.path.join(os.path.dirname(job_db_path), job_id + '.report_being_generated.' + report_type)
+    wf = open(tmp_flag_path, 'w')
+    wf.write(report_type)
+    wf.close()
     p = await asyncio.create_subprocess_shell(' '.join(run_args), stderr=asyncio.subprocess.PIPE)
     out, err = await p.communicate()
+    os.remove(tmp_flag_path)
     if report_type in report_generation_ps[job_id]:
         del report_generation_ps[job_id][report_type]
     if job_id in report_generation_ps and len(report_generation_ps[job_id]) == 0:
@@ -912,7 +923,7 @@ async def redirect_to_index (request):
         url = '/submit/nocache/index.html'
     return web.HTTPFound(url)
 
-async def load_live_modules ():
+async def load_live_modules (module_names=[]):
     global live_modules
     global live_mapper
     global include_live_modules
@@ -932,21 +943,24 @@ async def load_live_modules ():
     else:
         include_live_modules = []
         exclude_live_modules = []
-    live_modules = {}
-    cravat_conf = au.get_cravat_conf()
-    if 'genemapper' in cravat_conf:
-        default_mapper = cravat_conf['genemapper']
-    else:
-        default_mapper = 'hg38'
-    live_mapper = get_live_mapper(default_mapper)
+    if live_mapper is None:
+        cravat_conf = au.get_cravat_conf()
+        if 'genemapper' in cravat_conf:
+            default_mapper = cravat_conf['genemapper']
+        else:
+            default_mapper = 'hg38'
+        live_mapper = get_live_mapper(default_mapper)
     modules = au.get_local_module_infos(types=['annotator'])
     for module in modules:
-        if module.name in exclude_live_modules:
-            continue
-        if len(include_live_modules) > 0 and module.name not in include_live_modules:
-            continue
-        if 'secondary_inputs' in module.conf:
+        if module.name in live_modules:
             continue
+        if module.name not in module_names:
+            if module.name in exclude_live_modules:
+                continue
+            if len(include_live_modules) > 0 and module.name not in include_live_modules:
+                continue
+            if 'secondary_inputs' in module.conf:
+                continue
         annotator = get_live_annotator(module.name)
         if annotator is None:
             continue
@@ -1037,7 +1051,7 @@ async def get_live_annotation (queries):
         annotators = None
     global live_modules
     global mapper
-    if live_modules is None:
+    if len(live_modules) == 0:
         await load_live_modules()
         response = await live_annotate(input_data, annotators)
     else:
@@ -1119,7 +1133,7 @@ async def import_job (request):
 
 filerouter = FileRouter()
 VIEW_PROCESS = None
-live_modules = None
+live_modules = {}
 include_live_modules = None
 exclude_live_modules = None
 live_mapper = None
